@@ -1,6 +1,6 @@
 ;;; compile.el --- run compiler as inferior of Emacs, parse error messages.
 
-;; Copyright (C) 1985, 86, 87, 93, 94 Free Software Foundation, Inc.
+;; Copyright (C) 1985, 86, 87, 93, 94, 1995 Free Software Foundation, Inc.
 
 ;; Author: Roland McGrath <roland@prep.ai.mit.edu>
 ;; Maintainer: FSF
@@ -34,7 +34,7 @@
   "*List of hook functions run by `compilation-mode' (see `run-hooks').")
 
 ;;;###autoload
-(defconst compilation-window-height nil
+(defvar compilation-window-height nil
   "*Number of lines in a compilation window.  If nil, use Emacs default.")
 
 (defvar compilation-error-list nil
@@ -100,7 +100,7 @@ or when it is used with \\[next-error] or \\[compile-goto-error].")
 
 (defvar compilation-error-regexp-alist
   '(
-    ;; NOTE!  This first one is repeated in grep-regexp-alist, below.
+    ;; NOTE!  See also grep-regexp-alist, below.
 
     ;; 4.3BSD grep, cc, lint pass 1:
     ;; 	/usr/src/foo/foo.c(8): warning: w may be used before set
@@ -108,11 +108,21 @@ or when it is used with \\[next-error] or \\[compile-goto-error].")
     ;; 	foo.c:8: error message
     ;; or HP-UX 7.0 fc:
     ;; 	foo.f          :16    some horrible error message
-    ;;
+    ;; or GNU utilities with column (GNAT 1.82):
+    ;;   foo.adb:2:1: Unit name does not match file name
+    ;; 
     ;; We'll insist that the number be followed by a colon or closing
     ;; paren, because otherwise this matches just about anything
     ;; containing a number with spaces around it.
-    ("\n\\([^:( \t\n]+\\)[:(][ \t]*\\([0-9]+\\)[:) \t]" 1 2)
+    ("\n\
+\\([^:( \t\n]+\\)[:(][ \t]*\\([0-9]+\\)\\([) \t]\\|\
+:\\([^0-9\n]\\|\\([0-9]+:\\)\\)\\)" 1 2 5)
+
+    ;; Borland C++:
+    ;;  Error ping.c 15: Unable to open include file 'sys/types.h'
+    ;;  Warning ping.c 68: Call to function 'func' with no prototype
+    ("\n\\(Error\\|Warning\\) \\([^:( \t\n]+\\)\
+ \\([0-9]+\\)\\([) \t]\\|:[^0-9\n]\\)" 2 3)
 
     ;; 4.3BSD lint pass 2
     ;; 	strcmp: variable # of args. llib-lc(359)  ::  /usr/src/foo/foo.c(8)
@@ -127,7 +137,9 @@ or when it is used with \\[next-error] or \\[compile-goto-error].")
 
     ;; Ultrix 3.0 f77:
     ;;  fort: Severe: addstf.f, line 82: Missing operator or delimiter symbol
-    ("\nfort: [^:\n]*: \\([^ \n]*\\), line \\([0-9]+\\):" 1 2)
+    ;; Some SGI cc version:
+    ;;  cfe: Warning 835: foo.c, line 2: something
+    ("\n\\(cfe\\|fort\\): [^:\n]*: \\([^ \n]*\\), line \\([0-9]+\\):" 2 3)
     ;;  Error on line 3 of t.f: Execution error unclassifiable statement    
     ;; Unknown who does this:
     ;;  Line 45 of "foo.c": bloofel undefined
@@ -145,7 +157,9 @@ of[ \t]+\"?\\([^\":\n]+\\)\"?:" 3 2)
     ;;  File "foobar.ml", lines 5-8, characters 20-155: blah blah
     ;; Microtec mcc68k:
     ;;  "foo.c", line 32 pos 1; (E) syntax error; unexpected symbol: "lossage"
-    ("\"\\([^,\" \n\t]+\\)\", lines? \\([0-9]+\\)[:., -]" 1 2)
+    ;; GNAT (as of July 94): 
+    ;;  "foo.adb", line 2(11): warning: file name does not match ...
+    ("\"\\([^,\" \n\t]+\\)\", lines? \\([0-9]+\\)[:., (-]" 1 2)
 
     ;; MIPS RISC CC - the one distributed with Ultrix:
     ;;	ccom: Error: foo.c, line 2: syntax error
@@ -161,14 +175,33 @@ of[ \t]+\"?\\([^\":\n]+\\)\"?:" 3 2)
 
     ;; Lucid Compiler, lcc 3.x
     ;; E, file.cc(35,52) Illegal operation on pointers
-    ("[A-Z], \\([^(]*\\)(\\([0-9]+\\),[ \t]*\\([0-9]+\\)" 1 2 3)
+    ("\n[EW], \\([^(\n]*\\)(\\([0-9]+\\),[ \t]*\\([0-9]+\\)" 1 2 3)
 
+    ;; GNU messages with program name and optional column number.
+    ("\n[^0-9 \n\t:]+:[ \t]*\\([^ \n\t:]+\\):\
+\\([0-9]+\\):\\(\\([0-9]+\\)[: \t]\\)?" 1 2 4)
+
+    ;; Cray C compiler error messages
+    ("\n\\(cc\\| cft\\)-[0-9]+ c\\(c\\|f77\\): ERROR \\([^,\n]+, \\)* File = \\([^,\n]+\\), Line = \\([0-9]+\\)" 4 5)
+
+    ;; IBM C/C++ Tools 2.01:
+    ;;  foo.c(2:0) : informational EDC0804: Function foo is not referenced.
+    ;;  foo.c(3:8) : warning EDC0833: Implicit return statement encountered.
+    ;;  foo.c(5:5) : error EDC0350: Syntax error.
+    ("\n\\([^( \n\t]+\\)(\\([0-9]+\\):\\([0-9]+\\)) : " 1 2 3)
+
+    ;; Sun ada (VADS, Solaris):
+    ;;  /home3/xdhar/rcds_rc/main.a, line 361, char 6:syntax error: "," inserted
+    ("\n\\([^, ]+\\), line \\([0-9]+\\), char \\([0-9]+\\)[:., \(-]" 1 2 3)
     )
   "Alist that specifies how to match errors in compiler output.
-Each element has the form (REGEXP FILE-IDX LINE-IDX [COLUMN-IDX]).
+Each elt has the form (REGEXP FILE-IDX LINE-IDX [COLUMN-IDX FILE-FORMAT...])
 If REGEXP matches, the FILE-IDX'th subexpression gives the file name, and
 the LINE-IDX'th subexpression gives the line number.  If COLUMN-IDX is
-given, the COLUMN-IDX'th subexpression gives the column number on that line.")
+given, the COLUMN-IDX'th subexpression gives the column number on that line.
+If any FILE-FORMAT is given, each is a format string to produce a file name to
+try; %s in the string is replaced by the text matching the FILE-IDX'th
+subexpression.")
 
 (defvar compilation-read-command t
   "If not nil, M-x compile reads the compilation command to use.
@@ -183,7 +216,7 @@ Otherwise, it saves all modified buffers without asking.")
   "Regexp used to match grep hits.  See `compilation-error-regexp-alist'.")
 
 (defvar grep-command "grep -n "
-  "Last grep command used in \\{grep}; default for next grep.")
+  "Last grep command used in \\[grep]; default for next grep.")
 
 ;;;###autoload
 (defvar compilation-search-path '(nil)
@@ -204,14 +237,14 @@ You might also use mode hooks to specify it in certain modes, like this:
 				    (concat \"make -k \"
 					    buffer-file-name))))))")
 
-(defconst compilation-enter-directory-regexp
+(defvar compilation-enter-directory-regexp
   ": Entering directory `\\(.*\\)'$"
   "Regular expression matching lines that indicate a new current directory.
 This must contain one \\(, \\) pair around the directory name.
 
 The default value matches lines printed by the `-w' option of GNU Make.")
 
-(defconst compilation-leave-directory-regexp
+(defvar compilation-leave-directory-regexp
   ": Leaving directory `\\(.*\\)'$"
   "Regular expression matching lines that indicate restoring current directory.
 This may contain one \\(, \\) pair around the name of the directory
@@ -224,11 +257,23 @@ The default value matches lines printed by the `-w' option of GNU Make.")
   "Stack of previous directories for `compilation-leave-directory-regexp'.
 The head element is the directory the compilation was started in.")
 
+(defvar compilation-exit-message-function nil "\
+If non-nil, called when a compilation process dies to return a status message.
+This should be a function a two arguments as passed to a process sentinel
+\(see `set-process-sentinel\); it returns a cons (MESSAGE . MODELINE) of the
+strings to write into the compilation buffer, and to put in its mode line.")
+
 ;; History of compile commands.
 (defvar compile-history nil)
 ;; History of grep commands.
 (defvar grep-history nil)
 
+(defvar compilation-mode-font-lock-keywords
+  ;; This regexp needs a bit of rewriting.  What is the third grouping for?
+  '(("^\\([^ \n:]*:\\([0-9]+:\\)+\\)\\(.*\\)$" 1 font-lock-function-name-face))
+;;;  ("^\\([^\n:]*:\\([0-9]+:\\)+\\)\\(.*\\)$" 0 font-lock-keyword-face keep)
+  "Additional expressions to highlight in Compilation mode.")
+
 ;;;###autoload
 (defun compile (command)
   "Compile the program including the current buffer.  Default: run `make'.
@@ -238,6 +283,9 @@ with output going to the buffer `*compilation*'.
 You can then use the command \\[next-error] to find the next error message
 and move to the source code that caused it.
 
+Interactively, prompts for the command if `compilation-read-command' is
+non-nil; otherwise uses `compile-command'.  With prefix arg, always prompts.
+
 To run more than one compilation at once, start one and rename the
 \`*compilation*' buffer to some other name with \\[rename-buffer].
 Then start the next one.
@@ -246,7 +294,7 @@ The name used for the buffer is actually whatever is returned by
 the function in `compilation-buffer-name-function', so you can set that
 to a function that generates a unique name."
   (interactive
-   (if compilation-read-command
+   (if (or compilation-read-command current-prefix-arg)
        (list (read-from-minibuffer "Compile command: "
                                  compile-command nil nil
                                  '(compile-history . 1)))
@@ -255,6 +303,16 @@ to a function that generates a unique name."
   (save-some-buffers (not compilation-ask-about-save) nil)
   (compile-internal compile-command "No more errors"))
 
+;;; run compile with the default command line
+(defun recompile ()
+  "Re-compile the program including the current buffer."
+  (interactive)
+  (save-some-buffers (not compilation-ask-about-save) nil)
+  (compile-internal compile-command "No more errors"))
+
+;; The system null device. (Should reference NULL_DEVICE from C.)
+(defvar grep-null-device "/dev/null" "The system null device.")
+
 ;;;###autoload
 (defun grep (command-args)
   "Run grep, with user-specified args, and collect output in a buffer.
@@ -266,10 +324,23 @@ easily repeat a grep command."
   (interactive
    (list (read-from-minibuffer "Run grep (like this): "
 			       grep-command nil nil 'grep-history)))
-  (compile-internal (concat command-args " /dev/null")
-		    "No more grep hits" "grep"
-		    ;; Give it a simpler regexp to match.
-		    nil grep-regexp-alist))
+  (let ((buf (compile-internal (concat command-args " " grep-null-device)
+			       "No more grep hits" "grep"
+			       ;; Give it a simpler regexp to match.
+			       nil grep-regexp-alist)))
+    (save-excursion
+      (set-buffer buf)
+      (set (make-local-variable 'compilation-exit-message-function)
+	   (lambda (proc msg)
+	     (let ((code (process-exit-status proc)))
+	       (if (eq (process-status proc) 'exit)
+		   (cond ((zerop code)
+			  '("finished (matches found)\n" . "matched"))
+			 ((= code 1)
+			  '("finished with no matches found\n" . "no match"))
+			 (t
+			  (cons msg code)))
+		 (cons msg code))))))))
 
 (defun compile-internal (command error-message
 				 &optional name-of-mode parser regexp-alist
@@ -326,7 +397,9 @@ Returns the compilation buffer created."
 	;; will happen, and insert a `cd' command to indicate this.
 	(set-buffer outbuf)
 	(setq buffer-read-only nil)
+	(buffer-disable-undo (current-buffer))
 	(erase-buffer)
+	(buffer-enable-undo (current-buffer))
 	(setq default-directory thisdir)
 	(insert "cd " thisdir "\n" command "\n")
 	(set-buffer-modified-p nil))
@@ -339,7 +412,6 @@ Returns the compilation buffer created."
       (save-excursion
 	(set-buffer outbuf)
 	(compilation-mode)
-	(buffer-disable-undo (current-buffer))
 	;; (setq buffer-read-only t)  ;;; Non-ergonomic.
 	(set (make-local-variable 'compilation-parse-errors-function) parser)
 	(set (make-local-variable 'compilation-error-message) error-message)
@@ -350,20 +422,13 @@ Returns the compilation buffer created."
 	(setq mode-name name-of-mode)
 	(or (eq outwin (selected-window))
 	    (set-window-point outwin (point-min)))
-	(and compilation-window-height
-	     (= (window-width outwin) (frame-width))
-	     (let ((w (selected-window)))
-	       (unwind-protect
-		   (progn
-		     (select-window outwin)
-		     (enlarge-window (- compilation-window-height
-					(window-height))))
-		 (select-window w))))
+	(compilation-set-window-height outwin)
 	;; Start the compilation.
 	(if (fboundp 'start-process)
-	    (let ((proc (start-process-shell-command (downcase mode-name)
-						     outbuf
-						     command)))
+	    (let* ((process-environment (cons "EMACS=t" process-environment))
+		   (proc (start-process-shell-command (downcase mode-name)
+						      outbuf
+						      command)))
 	      (set-process-sentinel proc 'compilation-sentinel)
 	      (set-process-filter proc 'compilation-filter)
 	      (set-marker (process-mark proc) (point) outbuf)
@@ -371,16 +436,36 @@ Returns the compilation buffer created."
 		    (cons proc compilation-in-progress)))
 	  ;; No asynchronous processes available
 	  (message (format "Executing `%s'..." command))
+	  (sit-for 0) ;; Force redisplay
 	  (let ((status (call-process shell-file-name nil outbuf nil "-c"
 				      command))))
 	  (message (format "Executing `%s'...done" command)))))
     ;; Make it so the next C-x ` will use this buffer.
     (setq compilation-last-buffer outbuf)))
 
+;; Set the height of WINDOW according to compilation-window-height.
+(defun compilation-set-window-height (window)
+  (and compilation-window-height
+       (= (window-width window) (frame-width (window-frame window)))
+       ;; If window is alone in its frame, aside from a minibuffer,
+       ;; don't change its height.
+       (not (eq window (frame-root-window (window-frame window))))
+       ;; This save-excursion prevents us from changing the current buffer,
+       ;; which might not be the same as the selected window's buffer.
+       (save-excursion
+	 (let ((w (selected-window)))
+	   (unwind-protect
+	       (progn
+		 (select-window window)
+		 (enlarge-window (- compilation-window-height
+				    (window-height))))
+	     (select-window w))))))
+
 (defvar compilation-minor-mode-map
   (let ((map (make-sparse-keymap)))
     (define-key map [mouse-2] 'compile-mouse-goto-error)
     (define-key map "\C-c\C-c" 'compile-goto-error)
+    (define-key map "\C-m" 'compile-goto-error)
     (define-key map "\C-c\C-k" 'kill-compilation)
     (define-key map "\M-n" 'compilation-next-error)
     (define-key map "\M-p" 'compilation-previous-error)
@@ -393,6 +478,28 @@ Returns the compilation buffer created."
   (let ((map (cons 'keymap compilation-minor-mode-map)))
     (define-key map " " 'scroll-up)
     (define-key map "\^?" 'scroll-down)
+    ;; Set up the menu-bar
+    (define-key map [menu-bar compilation-menu]
+      (cons "Compile" (make-sparse-keymap "Compile")))
+
+    (define-key map [menu-bar compilation-menu compilation-mode-kill-compilation]
+      '("Stop Compilation" . kill-compilation))
+    (define-key map [menu-bar compilation-menu compilation-mode-separator2]
+      '("----" . nil))
+    (define-key map [menu-bar compilation-menu compilation-mode-first-error]
+      '("First Error" . first-error))
+    (define-key map [menu-bar compilation-menu compilation-mode-previous-error]
+      '("Previous Error" . previous-error))
+    (define-key map [menu-bar compilation-menu compilation-mode-next-error]
+      '("Next Error" . next-error))
+    (define-key map [menu-bar compilation-menu compilation-separator2]
+      '("----" . nil))
+    (define-key map [menu-bar compilation-menu compilation-mode-grep]
+      '("Grep" . grep))
+    (define-key map [menu-bar compilation-menu compilation-mode-recompile]
+      '("Recompile" . recompile))
+    (define-key map [menu-bar compilation-menu compilation-mode-compile]
+      '("Compile" . compile))
     map)
   "Keymap for compilation log buffers.
 `compilation-minor-mode-map' is a cdr of this.")
@@ -405,11 +512,13 @@ To kill the compilation, type \\[kill-compilation].
 
 Runs `compilation-mode-hook' with `run-hooks' (which see)."
   (interactive)
-  (fundamental-mode)
+  (kill-all-local-variables)
   (use-local-map compilation-mode-map)
   (setq major-mode 'compilation-mode
 	mode-name "Compilation")
   (compilation-setup)
+  (set (make-local-variable 'font-lock-defaults)
+       '(compilation-mode-font-lock-keywords t))
   (run-hooks 'compilation-mode-hook))
 
 ;; Prepare the buffer for the compilation parsing commands to work.
@@ -465,25 +574,29 @@ See `compilation-mode'."
 		    ;; Write something in the compilation buffer
 		    ;; and hack its mode line.
 		    (set-buffer buffer)
-		    (let ((buffer-read-only nil))
+		    (let ((buffer-read-only nil)
+			  (status (if compilation-exit-message-function
+				      (funcall compilation-exit-message-function
+					       proc msg)
+				    (cons msg (process-exit-status proc)))))
 		      (setq omax (point-max)
 			    opoint (point))
 		      (goto-char omax)
 		      ;; Record where we put the message, so we can ignore it
 		      ;; later on.
-		      (insert ?\n mode-name " " msg)
+		      (insert ?\n mode-name " " (car status))
 		      (forward-char -1)
 		      (insert " at " (substring (current-time-string) 0 19))
 		      (forward-char 1)
 		      (setq mode-line-process
-			    (concat ":"
-				    (symbol-name (process-status proc))))
+			    (format ":%s [%s]"
+				    (process-status proc) (cdr status)))
 		      ;; Since the buffer and mode line will show that the
 		      ;; process is dead, we can delete it now.  Otherwise it
 		      ;; will stay around until M-x list-processes.
 		      (delete-process proc)
 		      ;; Force mode line redisplay soon.
-		      (set-buffer-modified-p (buffer-modified-p)))
+		      (force-mode-line-update))
 		    (if (and opoint (< opoint omax))
 			(goto-char opoint))
 		    (if compilation-finish-function
@@ -495,13 +608,14 @@ See `compilation-mode'."
 (defun compilation-filter (proc string)
   "Process filter for compilation buffers.
 Just inserts the text, but uses `insert-before-markers'."
-  (save-excursion
-    (set-buffer (process-buffer proc))
-    (let ((buffer-read-only nil))
+  (if (buffer-name (process-buffer proc))
       (save-excursion
-	(goto-char (process-mark proc))
-	(insert-before-markers string)
-	(set-marker (process-mark proc) (point))))))
+	(set-buffer (process-buffer proc))
+	(let ((buffer-read-only nil))
+	  (save-excursion
+	    (goto-char (process-mark proc))
+	    (insert-before-markers string)
+	    (set-marker (process-mark proc) (point)))))))
 
 ;; Return the cdr of compilation-old-error-list for the error containing point.
 (defun compile-error-at-point ()
@@ -513,7 +627,9 @@ Just inserts the text, but uses `insert-before-markers'."
     errors))
 
 (defsubst compilation-buffer-p (buffer)
-  (assq 'compilation-error-list (buffer-local-variables buffer)))
+  (save-excursion
+    (set-buffer buffer)
+    (or compilation-minor-mode (eq major-mode 'compilation-mode))))
 
 (defun compilation-next-error (n)
   "Move point to the next error in the compilation buffer.
@@ -813,7 +929,20 @@ See variables `compilation-parse-errors-function' and
 			   (consp argp))))
 ;;;###autoload (define-key ctl-x-map "`" 'next-error)
 
-(defun compilation-next-error-locus (&optional move reparse)
+(defun previous-error ()
+  "Visit previous compilation error message and corresponding source code.
+This operates on the output from the \\[compile] command."
+  (interactive)
+  (next-error -1))
+
+(defun first-error ()
+  "Reparse the error message buffer and start at the first error
+Visit corresponding source code.
+This operates on the output from the \\[compile] command."
+  (interactive)
+  (next-error '(4)))
+
+(defun compilation-next-error-locus (&optional move reparse silent)
   "Visit next compilation error and return locus in corresponding source code.
 This operates on the output from the \\[compile] command.
 If all preparsed error messages have been processed,
@@ -826,120 +955,131 @@ marker at the location in the source code indicated by the error message.
 Optional first arg MOVE says how many error messages to move forwards (or
 backwards, if negative); default is 1.  Optional second arg REPARSE, if
 non-nil, says to reparse the error message buffer and reset to the first
-error (plus MOVE - 1).
+error (plus MOVE - 1).  If optional third argument SILENT is non-nil, return 
+nil instead of raising an error if there are no more errors.
 
 The current buffer should be the desired compilation output buffer."
   (or move (setq move 1))
   (compile-reinitialize-errors reparse nil (and (not reparse)
 						(if (< move 1) 0 (1- move))))
   (let (next-errors next-error)
-    (save-excursion
-      (set-buffer compilation-last-buffer)
-      ;; compilation-error-list points to the "current" error.
-      (setq next-errors 
-	    (if (> move 0)
-		(nthcdr (1- move)
-			compilation-error-list)
-	      ;; Zero or negative arg; we need to move back in the list.
-	      (let ((n (1- move))
-		    (i 0)
-		    (e compilation-old-error-list))
-		;; See how many cdrs away the current error is from the start.
-		(while (not (eq e compilation-error-list))
-		  (setq i (1+ i)
-			e (cdr e)))
-		(if (> (- n) i)
-		    (error "Moved back past first error")
-		  (nthcdr (+ i n) compilation-old-error-list))))
-	    next-error (car next-errors))
-      (while
-	  (if (null next-error)
-	      (progn
-		(and move (/= move 1)
-		     (error (if (> move 0)
-				"Moved past last error")
-			    "Moved back past first error"))
-		(compilation-forget-errors)
-		(error (concat compilation-error-message
-			       (and (get-buffer-process (current-buffer))
-				    (eq (process-status
-					 (get-buffer-process
-					  (current-buffer)))
-					'run)
-				    " yet"))))
-	    (setq compilation-error-list (cdr next-errors))
-	    (if (null (cdr next-error))
-		;; This error is boring.  Go to the next.
-		t
-	      (or (markerp (cdr next-error))
-		  ;; This error has a filename/lineno pair.
-		  ;; Find the file and turn it into a marker.
-		  (let* ((fileinfo (car (cdr next-error)))
-			 (buffer (compilation-find-file (cdr fileinfo)
-							(car fileinfo)
-							(car next-error))))
-		    (if (null buffer)
-			;; We can't find this error's file.
-			;; Remove all errors in the same file.
-			(progn
-			  (setq next-errors compilation-old-error-list)
-			  (while next-errors
-			    (and (consp (cdr (car next-errors)))
-				 (equal (car (cdr (car next-errors)))
-					fileinfo)
-				 (progn
-				   (set-marker (car (car next-errors)) nil)
-				   (setcdr (car next-errors) nil)))
-			    (setq next-errors (cdr next-errors)))
-			  ;; Look for the next error.
-			  t)
-		      ;; We found the file.  Get a marker for this error.
-		      ;; compilation-old-error-list is a buffer-local
-		      ;; variable, so we must be careful to extract its value
-		      ;; before switching to the source file buffer.
-		      (let ((errors compilation-old-error-list)
-			    (last-line (nth 1 (cdr next-error)))
-			    (column (nth 2 (cdr next-error))))
-			(set-buffer buffer)
-			(save-excursion
-			  (save-restriction
-			    (widen)
-			    (goto-line last-line)
-			    (if column
-				(move-to-column column)
-			      (beginning-of-line))
-			    (setcdr next-error (point-marker))
-			    ;; Make all the other error messages referring
-			    ;; to the same file have markers into the buffer.
-			    (while errors
-			      (and (consp (cdr (car errors)))
-				   (equal (car (cdr (car errors))) fileinfo)
-				   (let* ((this (nth 1 (cdr (car errors))))
-					  (column (nth 2 (cdr (car errors))))
-					  (lines (- this last-line)))
-				     (if (eq selective-display t)
-					 ;; When selective-display is t,
-					 ;; each C-m is a line boundary,
-					 ;; as well as each newline.
-					 (if (< lines 0)
-					     (re-search-backward "[\n\C-m]"
-								 nil 'end
-								 (- lines))
-					   (re-search-forward "[\n\C-m]"
-							      nil 'end
-							      lines))
-				       (forward-line lines))
-				     (if column
-					 (move-to-column column))
-				     (setq last-line this)
-				     (setcdr (car errors) (point-marker))))
-			      (setq errors (cdr errors)))))))))
-	      ;; If we didn't get a marker for this error, or this
-	      ;; marker's buffer was killed, go on to the next one.
-	      (or (not (markerp (cdr next-error)))
-		  (not (marker-buffer (cdr next-error))))))
-	(setq next-errors compilation-error-list
-	      next-error (car next-errors))))
+    (catch 'no-next-error
+      (save-excursion
+	(set-buffer compilation-last-buffer)
+	;; compilation-error-list points to the "current" error.
+	(setq next-errors 
+	      (if (> move 0)
+		  (nthcdr (1- move)
+			  compilation-error-list)
+		;; Zero or negative arg; we need to move back in the list.
+		(let ((n (1- move))
+		      (i 0)
+		      (e compilation-old-error-list))
+		  ;; See how many cdrs away the current error is from the start.
+		  (while (not (eq e compilation-error-list))
+		    (setq i (1+ i)
+			  e (cdr e)))
+		  (if (> (- n) i)
+		      (error "Moved back past first error")
+		    (nthcdr (+ i n) compilation-old-error-list))))
+	      next-error (car next-errors))
+	(while
+	    (if (null next-error)
+		(progn
+		  (and move (/= move 1)
+		       (error (if (> move 0)
+				  "Moved past last error")
+			      "Moved back past first error"))
+		  ;; Forget existing error messages if compilation has finished.
+		  (if (not (and (get-buffer-process (current-buffer))
+				(eq (process-status
+				     (get-buffer-process
+				      (current-buffer)))
+				    'run)))
+		      (compilation-forget-errors))
+		  (if silent
+		      (throw 'no-next-error nil)
+		    (error (concat compilation-error-message
+				   (and (get-buffer-process (current-buffer))
+					(eq (process-status
+					     (get-buffer-process
+					      (current-buffer)))
+					    'run)
+					" yet")))))
+	      (setq compilation-error-list (cdr next-errors))
+	      (if (null (cdr next-error))
+		  ;; This error is boring.  Go to the next.
+		  t
+		(or (markerp (cdr next-error))
+		    ;; This error has a filename/lineno pair.
+		    ;; Find the file and turn it into a marker.
+		    (let* ((fileinfo (car (cdr next-error)))
+			   (buffer (apply 'compilation-find-file
+					  (car next-error) fileinfo)))
+		      (if (null buffer)
+			  ;; We can't find this error's file.
+			  ;; Remove all errors in the same file.
+			  (progn
+			    (setq next-errors compilation-old-error-list)
+			    (while next-errors
+			      (and (consp (cdr (car next-errors)))
+				   (equal (car (cdr (car next-errors)))
+					  fileinfo)
+				   (progn
+				     (set-marker (car (car next-errors)) nil)
+				     (setcdr (car next-errors) nil)))
+			      (setq next-errors (cdr next-errors)))
+			    ;; Look for the next error.
+			    t)
+			;; We found the file.  Get a marker for this error.
+			;; compilation-old-error-list is a buffer-local
+			;; variable, so we must be careful to extract its value
+			;; before switching to the source file buffer.
+			(let ((errors compilation-old-error-list)
+			      (last-line (nth 1 (cdr next-error)))
+			      (column (nth 2 (cdr next-error))))
+			  (set-buffer buffer)
+			  (save-excursion
+			    (save-restriction
+			      (widen)
+			      (goto-line last-line)
+			      (if (and column (> column 0))
+				  ;; Columns in error msgs are 1-origin.
+				  (move-to-column (1- column))
+				(beginning-of-line))
+			      (setcdr next-error (point-marker))
+			      ;; Make all the other error messages referring
+			      ;; to the same file have markers into the buffer.
+			      (while errors
+				(and (consp (cdr (car errors)))
+				     (equal (car (cdr (car errors))) fileinfo)
+				     (let* ((this (nth 1 (cdr (car errors))))
+					    (column (nth 2 (cdr (car errors))))
+					    (lines (- this last-line)))
+				       (if (eq selective-display t)
+					   ;; When selective-display is t,
+					   ;; each C-m is a line boundary,
+					   ;; as well as each newline.
+					   (if (< lines 0)
+					       (re-search-backward "[\n\C-m]"
+								   nil 'end
+								   (- lines))
+					     (re-search-forward "[\n\C-m]"
+								nil 'end
+								lines))
+					 (forward-line lines))
+				       (if (and column (> column 1))
+					   (move-to-column (1- column))
+					 (beginning-of-line))
+				       (setq last-line this)
+				       (setcdr (car errors) (point-marker))))
+				(setq errors (cdr errors)))))))))
+		;; If we didn't get a marker for this error, or this
+		;; marker's buffer was killed, go on to the next one.
+		(or (not (markerp (cdr next-error)))
+		    (not (marker-buffer (cdr next-error))))))
+	  (setq next-errors compilation-error-list
+		next-error (car next-errors)))))
 
     ;; Skip over multiple error messages for the same source location,
     ;; so the next C-x ` won't go to an error in the same place.
@@ -969,9 +1109,13 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 
   ;; Show compilation buffer in other window, scrolled to this error.
   (let* ((pop-up-windows t)
-	 (w (display-buffer (marker-buffer (car next-error)))))
+	 ;; Use an existing window if it is in a visible frame.
+	 (w (or (get-buffer-window (marker-buffer (car next-error)) 'visible)
+		;; Pop up a window.
+		(display-buffer (marker-buffer (car next-error))))))
     (set-window-point w (car next-error))
-    (set-window-start w (car next-error))))
+    (set-window-start w (car next-error))
+    (compilation-set-window-height w)))
 
 ;; Find a buffer for file FILENAME.
 ;; Search the directories in compilation-search-path.
@@ -979,33 +1123,45 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 ;; current directory, which is passed in DIR.
 ;; If FILENAME is not found at all, ask the user where to find it.
 ;; Pop up the buffer containing MARKER and scroll to MARKER if we ask the user.
-(defun compilation-find-file (filename dir marker)
+(defun compilation-find-file (marker filename dir &rest formats)
+  (or formats (setq formats '("%s")))
   (let ((dirs compilation-search-path)
-	result name)
-    (while (and dirs (null result))
-      (setq name (expand-file-name filename (or (car dirs) dir))
-	    result (and (file-exists-p name)
-			(find-file-noselect name))
-	    dirs (cdr dirs)))
-    (or result
+	buffer thisdir fmts name)
+    (if (file-name-absolute-p filename)
+	;; The file name is absolute.  Use its explicit directory as
+	;; the first in the search path, and strip it from FILENAME.
+	(setq filename (abbreviate-file-name (expand-file-name filename))
+	      dirs (cons (file-name-directory filename) dirs)
+	      filename (file-name-nondirectory filename)))
+    ;; Now search the path.
+    (while (and dirs (null buffer))
+      (setq thisdir (or (car dirs) dir)
+	    fmts formats)
+      ;; For each directory, try each format string.
+      (while (and fmts (null buffer))
+	(setq name (expand-file-name (format (car fmts) filename) thisdir)
+	      buffer (and (file-exists-p name)
+			  (find-file-noselect name))
+	      fmts (cdr fmts)))
+      (setq dirs (cdr dirs)))
+    (or buffer
 	;; The file doesn't exist.
 	;; Ask the user where to find it.
 	;; If he hits C-g, then the next time he does
 	;; next-error, he'll skip past it.
-	(progn
-	  (let* ((pop-up-windows t)
-		 (w (display-buffer (marker-buffer marker))))
-	    (set-window-point w marker)
-	    (set-window-start w marker))
-	  (setq name
-		(expand-file-name
-		 (read-file-name
-		  (format "Find this error in: (default %s) "
-			  filename) dir filename t)))
-	  (if (file-directory-p name)
-	      (setq name (concat (file-name-as-directory name) filename)))
-	  (if (file-exists-p name)
-	      (find-file-noselect name))))))
+	(let* ((pop-up-windows t)
+	       (w (display-buffer (marker-buffer marker))))
+	  (set-window-point w marker)
+	  (set-window-start w marker)
+	  (let ((name (expand-file-name
+		       (read-file-name
+			(format "Find this error in: (default %s) "
+				filename)
+			dir filename t))))
+	    (if (file-directory-p name)
+		(setq name (expand-file-name filename name)))
+	    (and (file-exists-p name)
+		 (find-file-noselect name)))))))
 
 ;; Set compilation-error-list to nil, and unchain the markers that point to the
 ;; error messages and their text, so that they no longer slow down gap motion.
@@ -1112,9 +1268,14 @@ See variable `compilation-parse-errors-function' for the interface it uses."
       (setq subexpr (+ subexpr 1 (count-regexp-groupings (car (car alist)))))
       (setq alist (cdr alist)))
 
-    (setq orig default-directory)
-    (setq orig-expanded (file-truename orig))
-    (setq parent-expanded (expand-file-name "../" orig-expanded))
+    ;; Set up now the expanded, abbreviated directory variables
+    ;; that compile-abbreviate-directory will need, so we can
+    ;; compute them just once here.
+    (setq orig (abbreviate-file-name default-directory)
+	  orig-expanded (abbreviate-file-name
+			 (file-truename default-directory))
+	  parent-expanded (abbreviate-file-name
+			   (expand-file-name "../" orig-expanded)))
 
     (while (and (not found-desired)
 		;; We don't just pass LIMIT-SEARCH to re-search-forward
@@ -1161,7 +1322,7 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 			    (buffer-substring beg
 					      (match-end (+ leave-group
 							    1)))))))
-		     ;; The directory name in the "entering" message
+		     ;; The directory name in the "leaving" message
 		     ;; is a truename.  Try to convert it to a form
 		     ;; like what the user typed in.
 		     (setq dir
@@ -1205,6 +1366,7 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 			      (match-beginning (nth 2 alist))
 			      (match-end (nth 2 alist)))))
 		   (column (and (nth 3 alist)
+				(match-beginning (nth 3 alist))
 				(string-to-int
 				 (buffer-substring
 				  (match-beginning (nth 3 alist))
@@ -1219,14 +1381,20 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 		    ;; compile-abbreviate-directory).
 		    (file-name-absolute-p filename)
 		    (setq filename (concat comint-file-name-prefix filename)))
-	       (setq filename (cons default-directory filename))
+	       (setq filename (cons filename (cons default-directory
+						   (nthcdr 4 alist))))
+				     
 
 	       ;; Locate the erring file and line.
 	       ;; Cons a new elt onto compilation-error-list,
 	       ;; giving a marker for the current compilation buffer
 	       ;; location, and the file and line number of the error.
 	       (save-excursion
-		 (beginning-of-line 1)
+		 ;; Save as the start of the error the beginning of the
+		 ;; line containing the match unless the match starts at a
+		 ;; newline, in which case the beginning of the next line.
+		 (goto-char beginning-of-match)
+		 (forward-line (if (eolp) 1 0))
 		 (let ((this (cons (point-marker)
 				   (list filename linenum column))))
 		   ;; Don't add the same source line more than once.
@@ -1287,6 +1455,10 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 ;; Those two args could be computed here, but we run faster by
 ;; having the caller compute them just once.
 (defun compile-abbreviate-directory (dir orig orig-expanded parent-expanded)
+  ;; Apply canonical abbreviations to DIR first thing.
+  ;; Those abbreviations are already done in the other arguments passed.
+  (setq dir (abbreviate-file-name dir))
+
   ;; Check for a comint-file-name-prefix and prepend it if appropriate.
   ;; (This is very useful for compilation-minor-mode in an rlogin-mode
   ;; buffer.)
