@@ -1,6 +1,6 @@
 ;;; telnet.el --- run a telnet session from within an Emacs buffer
 
-;;; Copyright (C) 1985, 1988, 1992 Free Software Foundation, Inc.
+;;; Copyright (C) 1985, 1988, 1992, 1994 Free Software Foundation, Inc.
 
 ;; Author: William F. Schelter
 ;; Maintainer: FSF
@@ -51,7 +51,7 @@
 
 (defvar telnet-new-line "\r")
 (defvar telnet-mode-map nil)
-(defvar telnet-prompt-pattern "^[^#$%>]*[#$%>] *")
+(defvar telnet-prompt-pattern "^[^#$%>\n]*[#$%>] *")
 (defvar telnet-replace-c-g nil)
 (make-variable-buffer-local
  (defvar telnet-remote-echoes t
@@ -60,8 +60,11 @@
  (defvar telnet-interrupt-string "\C-c" "String sent by C-c."))
 
 (defvar telnet-count 0
-  "Number of output strings read from the telnet process
-while looking for the initial password.")
+  "Number of output strings from telnet process while looking for password.")
+(make-variable-buffer-local 'telnet-count)
+
+(defvar telnet-program "telnet"
+  "Program to run to open a telnet connection.")
 
 (defvar telnet-initial-count -50
   "Initial value of `telnet-count'.  Should be set to the negative of the
@@ -71,7 +74,7 @@ number of terminal writes telnet will make setting up the host connection.")
   "Maximum value `telnet-count' can have.
 After this many passes, we stop looking for initial setup data.
 Should be set to the number of terminal writes telnet will make
-rejecting one login and prompting for the again for a username and password.")
+rejecting one login and prompting again for a username and password.")
 
 (defun telnet-interrupt-subjob ()
   (interactive)
@@ -93,7 +96,7 @@ rejecting one login and prompting for the again for a username and password.")
 ; initialization on first load.
 (if telnet-mode-map
     nil
-  (setq telnet-mode-map (copy-keymap comint-mode-map))
+  (setq telnet-mode-map (nconc (make-sparse-keymap) comint-mode-map))
   (define-key telnet-mode-map "\C-m" 'telnet-send-input)
 ;  (define-key telnet-mode-map "\C-j" 'telnet-send-input)
   (define-key telnet-mode-map "\C-c\C-q" 'send-process-next-char)
@@ -110,7 +113,7 @@ rejecting one login and prompting for the again for a username and password.")
 	((string-match "tops-20" string) ;;maybe add telnet-replace-c-g
 	 (setq telnet-prompt-pattern  "[@>]*"))
 	((string-match "its" string)
-	 (setq telnet-prompt-pattern  "^[^*>]*[*>] *"))
+	 (setq telnet-prompt-pattern  "^[^*>\n]*[*>] *"))
 	((string-match "explorer" string)  ;;explorer telnet needs work
 	 (setq telnet-replace-c-g ?\n))))
   (setq comint-prompt-regexp telnet-prompt-pattern))
@@ -122,10 +125,9 @@ rejecting one login and prompting for the again for a username and password.")
 	 (error "No such host."))
 	((string-match "passw" string)
 	 (telnet-filter proc string)
-	 (let* ((echo-keystrokes 0)
-		(password (read-password)))
-	   (setq telnet-count 0)
-	   (send-string proc (concat password telnet-new-line))))
+	 (setq telnet-count 0)
+	 (send-string proc (concat (comint-read-noecho "Password: " t)
+				   telnet-new-line)))
 	(t (telnet-check-software-type-initialize string)
 	   (telnet-filter proc string)
 	   (cond ((> telnet-count telnet-maximum-count)
@@ -139,28 +141,30 @@ rejecting one login and prompting for the again for a username and password.")
   (comint-send-string proc telnet-new-line))
 
 (defun telnet-filter (proc string)
-  (let ((at-end
-	 (and (eq (process-buffer proc) (current-buffer))
-	      (= (point) (point-max)))))
-    (save-excursion
-      (set-buffer (process-buffer proc))
+  (save-excursion
+    (set-buffer (process-buffer proc))
+    (let* ((last-insertion (marker-position (process-mark proc)))
+	   (delta (- (point) last-insertion))
+	   (ie (and comint-last-input-end
+		    (marker-position comint-last-input-end)))
+	   (w (get-buffer-window (current-buffer)))
+	   (ws (and w (window-start w))))
+      (goto-char last-insertion)
+      (insert-before-markers string)
+      (set-marker (process-mark proc) (point))
+      (if ws (set-window-start w ws t))
+      (if ie (set-marker comint-last-input-end ie))
+      (while (progn (skip-chars-backward "^\C-m" last-insertion)
+		    (> (point) last-insertion))
+	(delete-region (1- (point)) (point)))
       (goto-char (process-mark proc))
-      (let ((now (point)))
-	;; Insert STRING, omitting all C-m characters.
-	(let ((index 0) c-m)
-	  (while (setq c-m (string-match "\C-m" string index))
-	    (insert-before-markers (substring string index c-m))
-	    (setq index (1+ c-m)))
-	  (insert-before-markers (substring string index)))
-	(and telnet-replace-c-g
-	     (subst-char-in-region now (point) ?\C-g telnet-replace-c-g)))
-;      (if (and (integer-or-marker-p last-input-start)
-;	       (marker-position last-input-start)
-;	       telnet-remote-echoes)
-;	  (delete-region last-input-start last-input-end))
-      )
-    (if at-end
-	(goto-char (point-max)))))
+      (and telnet-replace-c-g
+	   (subst-char-in-region last-insertion (point) ?\C-g
+				 telnet-replace-c-g t))
+      ;; If point is after the insertion place, move it
+      ;; along with the text.
+      (if (> delta 0)
+	  (goto-char (+ (process-mark proc) delta))))))
 
 (defun telnet-send-input ()
   (interactive)
@@ -170,22 +174,27 @@ rejecting one login and prompting for the again for a username and password.")
       (delete-region comint-last-input-start
 		     comint-last-input-end)))
 
+;;;###autoload (add-hook 'same-window-regexps "\\*telnet-.*\\*\\(\\|<[0-9]+>\\)")
+
 ;;;###autoload
 (defun telnet (host)
   "Open a network login connection to host named HOST (a string).
-Communication with HOST is recorded in a buffer *HOST-telnet*.
+Communication with HOST is recorded in a buffer `*telnet-HOST*'.
 Normally input is edited in Emacs and sent a line at a time."
   (interactive "sOpen telnet connection to host: ")
-  (let* ((name (concat host "-telnet" ))
-	 (buffer (get-buffer (concat "*" name "*"))))
+  (let* ((comint-delimiter-argument-list '(?\  ?\t))
+         (name (concat "telnet-" (comint-arguments host 0 nil) ))
+	 (buffer (get-buffer (concat "*" name "*")))
+	 process)
     (if (and buffer (get-buffer-process buffer))
-	(switch-to-buffer (concat "*" name "*"))
-      (switch-to-buffer (make-comint name "telnet"))
-      (set-process-filter (get-process name) 'telnet-initial-filter)
+	(pop-to-buffer (concat "*" name "*"))
+      (pop-to-buffer (make-comint name telnet-program))
+      (setq process (get-buffer-process (current-buffer)))
+      (set-process-filter process 'telnet-initial-filter)
       ;; Don't send the `open' cmd till telnet is ready for it.
-      (accept-process-output (get-process name))
+      (accept-process-output process)
       (erase-buffer)
-      (send-string  name (concat "open " host "\n"))
+      (send-string process (concat "open " host "\n"))
       (telnet-mode)
       (setq comint-input-sender 'telnet-simple-send)
       (setq telnet-count telnet-initial-count))))
@@ -207,27 +216,21 @@ Data is sent to the remote host when RET is typed.
   (use-local-map telnet-mode-map)
   (run-hooks 'telnet-mode-hook))
 
+;;;###autoload (add-hook 'same-window-regexps "\\*rsh-[^-]*\\*\\(\\|<[0-9]*>\\)")
+
 ;;;###autoload
 (defun rsh (host)
   "Open a network login connection to host named HOST (a string).
-Communication with HOST is recorded in a buffer *HOST-rsh*.
+Communication with HOST is recorded in a buffer `*rsh-HOST*'.
 Normally input is edited in Emacs and sent a line at a time."
   (interactive "sOpen rsh connection to host: ")
   (require 'shell)
-  (let ((name (concat host "-rsh" )))
-    (switch-to-buffer (make-comint name "rsh" nil host))
+  (let ((name (concat "rsh-" host )))
+    (pop-to-buffer (make-comint name remote-shell-program nil host))
     (set-process-filter (get-process name) 'telnet-initial-filter)
     (telnet-mode)
     (setq telnet-count -16)))
 
-(defun read-password ()
-  (let ((answ "") tem)
-    (message "Reading password...")
-    (while (prog1 (not (memq (setq tem (read-char)) '(?\C-m ?\n ?\C-g)))
-	     (setq quit-flag nil))
-      (setq answ (concat answ (char-to-string tem))))
-    answ))
-
 (provide 'telnet)
 
 ;;; telnet.el ends here
