@@ -39,6 +39,18 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #ifndef MSDOS
 #define MSDOS
 #endif
+
+#ifdef __GO32__
+#ifndef __DJGPP__
+#define __DJGPP__ 1	/* V2 defines __DJGPP__ == 2 */
+#else
+You lose; /* Emacs for DOS must be compiled with DJGPP V1 */
+#endif
+#else
+You lose; /* Emacs for DOS must be compiled with DJGPP */
+#endif
+
+#define DOS_NT	/* MSDOS or WINDOWSNT */
 #undef BSD
 #undef VMS
 
@@ -47,7 +59,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define SYSTEM_TYPE "ms-dos"
 
-#define SYMS_SYSTEM syms_of_dosfns()
+#define SYMS_SYSTEM syms_of_dosfns();syms_of_msdos()
 
 /* NOMULTIPLEJOBS should be defined if your system's shell
  does not have "job control" (the ability to stop a program,
@@ -80,19 +92,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* #define FIRST_PTY_LETTER 'a' */
 
-/*
- *	Define HAVE_TIMEVAL if the system supports the BSD style clock values.
- *	Look in <sys/time.h> for a timeval structure.
- */
-
-#define HAVE_TIMEVAL
-
-/*
- *	Define HAVE_SELECT if the system supports the `select' system call.
- */
-
-/* #define HAVE_SELECT */
-
 /*
  *	Define HAVE_PTYS if the system supports pty devices.
  */
@@ -149,13 +148,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    your system and must be used only through an encapsulation
    (Which you should place, by convention, in sysdep.c).  */
 
-/* Some compilers tend to put everything declared static
-   into the initialized data area, which becomes pure after dumping Emacs.
-   On these systems, you must #define static as nothing to foil this.
-   Note that emacs carefully avoids static vars inside functions.  */
-
-/* #define static */
-
 /* we use djgcc's malloc */
 /* #define SYSTEM_MALLOC */
 /* setjmp and longjmp can safely replace _setjmp and _longjmp,
@@ -166,9 +158,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define NO_MODE_T
 
-/* we can use dj's getpagesize() */
-#define HAVE_GETPAGESIZE
-
 /* New chdir () routine. */
 #ifdef chdir
 #undef chdir
@@ -182,7 +171,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define ORDINARY_LINK
 
-/* command.com does not under stand `...` so we define this.  */
+/* command.com does not understand `...` so we define this.  */
 #define LIB_GCC -Lgcc
 #define DONT_NEED_ENVIRON
 #define SEPCHAR ';'
@@ -206,27 +195,44 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* When $TERM is "internal" then this is substituted:  */
 #define INTERNAL_TERMINAL "pc|bios|IBM PC with colour display:\
-:co#80:li#25:km:\
-:cm=\E@%.%.:\
-:do=^J:le=^H:up=\EU:ri=\ER:\
-:ti=\EA\027:te=\EA\007\EE:\
-:so=\EA\077:se=\EA\027:\
-:ms:mb=\EX\200:md=\EX\010:mk=\EA\161:me=\EA\027:\
-:cl=\EC:ce=\EE:\
-:vb=\EB\140:bl=\007:"
-#define fflush internal_flush
+:co#80:li#25:km:ms:cm=<CM>:cl=<CL>:ce=<CE>:"
 
 /* Define this to a function (Fdowncase, Fupcase) if your file system
    likes that */
 #define FILE_SYSTEM_CASE Fdowncase
 
+/* Define this to be the separator between devices and paths */
+#define DEVICE_SEP ':'
+
+/* We'll support either convention on MSDOG.  */
+#define IS_DIRECTORY_SEP(_c_) ((_c_) == '/' || (_c_) == '\\')
+#define IS_ANY_SEP(_c_) (IS_DIRECTORY_SEP (_c_) || IS_DEVICE_SEP (_c_))
+
+/* Call init_gettimeofday when TZ changes.  */
+#define LOCALTIME_CACHE
+#define tzset init_gettimeofday
+
 /* bcopy under djgpp is quite safe */
 #define GAP_USE_BCOPY
 #define BCOPY_UPWARD_SAFE 1
 #define BCOPY_DOWNWARD_SAFE 1
 
-/* We need a little extra space, see ../../lisp/loadup.el */
-#define PURESIZE 240000
+/* Mode line description of a buffer's type.  */
+#define MODE_LINE_BINARY_TEXT(buf) (NILP(buf->buffer_file_type) ? "T" : "B")
 
 /* We have (the code to control) a mouse.  */
 #define HAVE_MOUSE
+
+/* We have support for faces.  */
+#define HAVE_FACES
+
+/* Define one of these for easier conditionals.  */
+#ifdef HAVE_X_WINDOWS
+/* We need a little extra space, see ../../lisp/loadup.el */
+#define SYSTEM_PURESIZE_EXTRA 15000
+#define HAVE_X11R5
+#define LIBX11_SYSTEM -lxext -lsys
+#else
+/* We need a little extra space, see ../../lisp/loadup.el */
+#define SYSTEM_PURESIZE_EXTRA 85000
+#endif
